@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,28 +10,32 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
-from pyramid.exceptions import NotFound
+"""PyAMS_content.component.file module
+
+This module provides a custom file view to check parents publication.
+"""
+
+__docformat__ = 'restructuredtext'
+
+from pyramid.httpexceptions import HTTPNotFound
 from pyramid.location import lineage
 from pyramid.view import view_config
 
 from pyams_file.interfaces import IFile
-from pyams_file.views.file import FileView
-from pyams_skin.layer import IPyAMSUserLayer
-from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
+from pyams_file.skin.view import FileView
+from pyams_layer.interfaces import IPyAMSUserLayer
+from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
 from pyams_workflow.interfaces import IWorkflowPublicationInfo
 
 
-__docformat__ = 'restructuredtext'
-
-
-@view_config(context=IFile, request_type=IPyAMSUserLayer)
-def ProtectedFileView(request):
+@view_config(context=IFile,
+             request_type=IPyAMSUserLayer)
+def ProtectedFileView(request):  # pylint: disable=invalid-name
     """Protected file view"""
     context = request.context
-    if not request.has_permission(VIEW_SYSTEM_PERMISSION, context=context):  # authenticated operator
+    if not request.has_permission(VIEW_SYSTEM_PERMISSION, context=context):  # authenticated
         for parent in lineage(context):
             publication_info = IWorkflowPublicationInfo(parent, None)
             if (publication_info is not None) and not publication_info.is_visible(request):
-                raise NotFound()
-
+                raise HTTPNotFound()
     return FileView(request)
