@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,16 +10,14 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
-"""PyAMS_*** module
+"""PyAMS_content.component.illustration.portlet module
 
+This module provides illustration portlet.
 """
 
-from zope.schema.fieldproperty import FieldProperty
-
 from pyams_content.component.illustration.portlet.interfaces import IIllustrationPortletSettings
 from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
 from pyams_utils.factory import factory_config
-from pyams_utils.interfaces import VIEW_PERMISSION
 
 
 __docformat__ = 'restructuredtext'
@@ -28,22 +26,20 @@ from pyams_content import _
 
 
 ILLUSTRATION_PORTLET_NAME = 'pyams_content.portlet.illustration'
+ILLUSTRATION_ICON_CLASS = 'far fa-image'
 
 
-@factory_config(provided=IIllustrationPortletSettings)
+@factory_config(IIllustrationPortletSettings)
 class IllustrationPortletSettings(PortletSettings):
-    """Illustration portlet persistent settings"""
-
-    display = FieldProperty(IIllustrationPortletSettings['display'])
+    """Illustration portlet settings"""
 
 
-@portlet_config(permission=VIEW_PERMISSION)
+@portlet_config(permission=None)
 class IllustrationPortlet(Portlet):
     """Illustration portlet"""
 
     name = ILLUSTRATION_PORTLET_NAME
     label = _("Illustration")
 
-    toolbar_css_class = 'fa fa-fw fa-2x fa-file-image-o'
-
     settings_factory = IIllustrationPortletSettings
+    toolbar_css_class = ILLUSTRATION_ICON_CLASS
