@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,23 +10,31 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
-__docformat__ = 'restructuredtext'
+"""PyAMS_content.component.illustration.thesaurus module
 
-from pyams_content.component.illustration import IBasicIllustrationTarget
+This module provides a custom thesaurus extension which allows to assign an
+illustration on any thesaurus term.
+"""
+
+from pyams_content.component.illustration.interfaces import IBaseIllustrationTarget
 from pyams_thesaurus.interfaces.extension import IThesaurusTermExtension
 from pyams_utils.registry import utility_config
 
+
+__docformat__ = 'restructuredtext'
+
 from pyams_content import _
 
 
-@utility_config(name='illustration', provides=IThesaurusTermExtension)
-class IllustrationThesaurusExtension(object):
-    """Illustration thesaurus extension"""
+@utility_config(name='illustration',
+                provides=IThesaurusTermExtension)
+class IllustrationThesaurusTermExtension:
+    """Illustration thesaurus term extension"""
 
     label = _("Illustration")
     weight = 20
 
-    target_interface = IBasicIllustrationTarget
-    target_view = 'illustration-dialog.html'
+    icon_css_class = 'far fa-image'
 
-    icon = '<i class="fa fa-fw fa-file-image-o"></i>'
+    target_interface = IBaseIllustrationTarget
+    target_view = 'illustration-dialog.html'
