@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,12 +10,8 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
+"""PyAMS_content.component.video.zmi module
+
+"""
+
 __docformat__ = 'restructuredtext'
-
-# import standard library
-
-# import interfaces
-
-# import packages
-
-
