@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,17 +10,20 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
+"""PyAMS_content.reference.interfaces module
+
+"""
+
+from zope.container.constraints import containers, contains
+from zope.container.interfaces import IContainer
+from zope.interface import Interface
+
+from pyams_content.interfaces import IBaseContent, REFERENCE_MANAGER_ROLE
+from pyams_security.schema import PrincipalsSetField
+
 __docformat__ = 'restructuredtext'
 
-
-# import standard library
-
-# import interfaces
-from pyams_content.interfaces import IBaseContent
-from zope.container.interfaces import IContainer
-
-# import packages
-from zope.container.constraints import containers, contains
+from pyams_content import _
 
 
 class IReferenceInfo(IBaseContent):
@@ -36,6 +39,18 @@ class IReferenceTable(IBaseContent):
     contains(IReferenceInfo)
 
 
+REFERENCE_TABLE_ROLES = 'pyams_content.reference.roles'
+
+
+class IReferenceTableRoles(Interface):
+    """Reference table roles interface"""
+
+    managers = PrincipalsSetField(title=_("Table managers"),
+                                  description=_("Table managers can handle all table contents"),
+                                  role_id=REFERENCE_MANAGER_ROLE,
+                                  required=False)
+
+
 class IReferenceManager(IBaseContent, IContainer):
     """References tables container"""
 
