@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2017 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,10 +10,9 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
+"""PyAMS_content.shared.common.portlet module
+
+This is the base module for common shared contents portlets.
+"""
+
 __docformat__ = 'restructuredtext'
-
-# import standard library
-
-# import interfaces
-
-# import packages
