@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,59 +10,73 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
+"""PyAMS_content.workflow.basic module
+
+This module defines a basic PyAMS content workflow.
+"""
+
 from datetime import datetime, timedelta, timezone
 
 from zope.copy import copy
 from zope.interface import implementer
-from zope.intid import IIntIds
+from zope.intid.interfaces import IIntIds
 from zope.location import locate
 from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
 
-from pyams_content.interfaces import CREATE_VERSION_PERMISSION, MANAGER_ROLE, \
-    MANAGE_CONTENT_PERMISSION, MANAGE_SITE_ROOT_PERMISSION, OWNER_ROLE, PILOT_ROLE, \
-    PUBLISH_CONTENT_PERMISSION, READER_ROLE, WEBMASTER_ROLE
-from pyams_content.shared.common.interfaces import IContributorRestrictions, IManagerRestrictions
-from pyams_content.shared.common.interfaces import IWfSharedContentRoles
-from pyams_content.workflow import ContentArchivingTask, ContentPublishingTask
+from pyams_content.interfaces import CREATE_VERSION_PERMISSION, MANAGER_ROLE, MANAGE_CONTENT_PERMISSION, \
+    MANAGE_SITE_ROOT_PERMISSION, OWNER_ROLE, PILOT_ROLE, PUBLISH_CONTENT_PERMISSION, READER_ROLE, WEBMASTER_ROLE
+from pyams_content.shared.common.interfaces import BASIC_CONTENT_WORKFLOW, IContributorRestrictions, \
+    IManagerRestrictions, IWfSharedContentRoles
+from pyams_content.workflow import CANCELED, ContentArchivingTask, ContentPublishingTask, prepublished_to_published
 from pyams_content.workflow.interfaces import IBasicWorkflow
-from pyams_scheduler.interfaces import IDateTaskScheduling, IScheduler
-from pyams_security.interfaces import INTERNAL_USER_ID, IRoleProtectedObject
+from pyams_scheduler.interfaces import IScheduler
+from pyams_scheduler.interfaces.task import IDateTaskScheduling, SCHEDULER_TASK_DATE_MODE
+from pyams_security.interfaces import IRoleProtectedObject
 from pyams_sequence.interfaces import ISequentialIdInfo
 from pyams_utils.adapter import ContextAdapter, adapter_config
 from pyams_utils.date import format_datetime
 from pyams_utils.registry import get_current_registry, get_utility, query_utility, utility_config
 from pyams_utils.request import check_request
 from pyams_utils.timezone import gmtime
-from pyams_workflow.interfaces import IWorkflow, IWorkflowInfo, IWorkflowPublicationInfo, \
-    IWorkflowState, IWorkflowStateLabel, IWorkflowVersions, ObjectClonedEvent, SYSTEM
+from pyams_workflow.interfaces import IWorkflow, IWorkflowInfo, IWorkflowPublicationInfo, IWorkflowState, \
+    IWorkflowStateLabel, IWorkflowVersions, ObjectClonedEvent, SYSTEM_TRANSITION
 from pyams_workflow.workflow import Transition, Workflow
 
-
 __docformat__ = 'restructuredtext'
 
 from pyams_content import _
 
 
+#
+# Workflow states
+#
+
 DRAFT = 'draft'
 PRE_PUBLISHED = 'pre-published'
 PUBLISHED = 'published'
 ARCHIVED = 'archived'
 DELETED = 'deleted'
 
-STATES_IDS = (DRAFT,
-              PRE_PUBLISHED,
-              PUBLISHED,
-              ARCHIVED,
-              DELETED)
+STATES_IDS = (
+    DRAFT,
+    PRE_PUBLISHED,
+    PUBLISHED,
+    ARCHIVED,
+    DELETED
+)
 
-STATES_LABELS = (_("Draft"),
-                 _("Published (waiting)"),
-                 _("Published"),
-                 _("Archived"),
-                 _("Deleted"))
+STATES_LABELS = (
+    _("Draft"),
+    _("Published (waiting)"),
+    _("Published"),
+    _("Archived"),
+    _("Deleted")
+)
 
-STATES_VOCABULARY = SimpleVocabulary([SimpleTerm(STATES_IDS[i], title=t)
-                                      for i, t in enumerate(STATES_LABELS)])
+STATES_VOCABULARY = SimpleVocabulary([
+    SimpleTerm(STATES_IDS[i], title=t)
+    for i, t in enumerate(STATES_LABELS)
+])
 
 STATES_HEADERS = {
     DRAFT: _("draft created"),
@@ -71,25 +85,27 @@ STATES_HEADERS = {
     ARCHIVED: _("archived")
 }
 
+
 UPDATE_STATES = (DRAFT, )
 '''Default state available to contributors in update mode'''
 
 READONLY_STATES = (ARCHIVED, DELETED)
 '''Retired and archived contents can't be modified'''
 
-PROTECTED_STATES = (PRE_PUBLISHED, PUBLISHED, )
+PROTECTED_STATES = (PRE_PUBLISHED, PUBLISHED)
 '''Protected states are available to webmasters in update mode'''
 
 MANAGER_STATES = ()
-'''No custom state available to managers!'''
+'''Only managers can update proposed contents (if their restrictions apply)'''
 
 PUBLISHED_STATES = (PRE_PUBLISHED, PUBLISHED)
-'''Pre-published and published states are marked as published'''
+'''Contents in published states are pre-published, published or waiting for retiring'''
 
 VISIBLE_STATES = (PUBLISHED, )
-'''Only published state is visible in front-office'''
+'''Contents in visible states are visible in front-office'''
 
 WAITING_STATES = ()
+'''Contents in waiting states are waiting for a manager action'''
 
 RETIRED_STATES = ()
 
@@ -100,77 +116,75 @@ ARCHIVED_STATES = (ARCHIVED, )
 # Workflow conditions
 #
 
-def can_manage_content(wf, context):
-    """Check if a manager can handle content"""
-    request = check_request()
-    # grant access to webmaster
-    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context=context):
-        return True
-    # local content managers can manage content
-    principal_id = request.principal.id
-    if principal_id in {INTERNAL_USER_ID} | context.managers:
-        return True
-    # shared tool managers can manage content if restrictions apply
-    restrictions = IManagerRestrictions(context).get_restrictions(principal_id)
-    return restrictions and restrictions.check_access(context,
-                                                      permission=PUBLISH_CONTENT_PERMISSION,
-                                                      request=request)
-
-
 def can_create_new_version(wf, context):
     """Check if we can create a new version"""
     # can't create new version when previous draft already exists
     versions = IWorkflowVersions(context)
-    if versions.has_version(DRAFT) or versions.has_version(PRE_PUBLISHED):
+    if (versions.has_version(DRAFT) or
+            versions.has_version(PRE_PUBLISHED)):
         return False
     request = check_request()
     # grant access to webmaster
-    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context=context):
+    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
         return True
-    # grant access to owner, creator and local contributors
+    # grant access to owner, creator and local contributors and managers
+    roles = IWfSharedContentRoles(context)
     principal_id = request.principal.id
-    if principal_id in context.owner | {context.creator} | context.contributors:
+    if principal_id in {context.creator} | roles.owner | roles.contributors | roles.managers:
         return True
     # grant access to allowed contributors
-    restrictions = IContributorRestrictions(context).get_restrictions(principal_id)
-    if restrictions and restrictions.check_access(context,
-                                                  permission=MANAGE_CONTENT_PERMISSION,
-                                                  request=request):
-        return True
-    # grant access to local content managers
-    if principal_id in context.managers:
+    restrictions = IContributorRestrictions(context)
+    if restrictions and restrictions.can_access(context,
+                                                permission=MANAGE_CONTENT_PERMISSION,
+                                                request=request):
         return True
     # grant access to shared tool managers if restrictions apply
-    restrictions = IManagerRestrictions(context).get_restrictions(principal_id)
-    return restrictions and restrictions.check_access(context,
-                                                      permission=CREATE_VERSION_PERMISSION,
-                                                      request=request)
+    restrictions = IManagerRestrictions(context)
+    return restrictions and restrictions.can_access(context,
+                                                    permission=CREATE_VERSION_PERMISSION,
+                                                    request=request)
+
+
+def can_manage_content(wf, context):
+    """Check if a manager can handle content"""
+    request = check_request()
+    # grant access to webmaster
+    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
+        return True
+    # local content managers can manage content
+    roles = IWfSharedContentRoles(context)
+    principal_id = request.principal.id
+    if principal_id in roles.owner | roles.managers:
+        return True
+    # shared tool managers can manage content if restrictions apply
+    restrictions = IManagerRestrictions(context)
+    return restrictions and restrictions.can_access(context,
+                                                    permission=PUBLISH_CONTENT_PERMISSION,
+                                                    request=request)
 
 
 def can_delete_version(wf, context):
     """Check if we can delete a draft version"""
     request = check_request()
     # grant access to webmaster
-    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context=context):
+    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
         return True
-    # grant access to owner, creator and local contributors
+    # grant access to owner, creator and local contributors and managers
+    roles = IWfSharedContentRoles(context)
     principal_id = request.principal.id
-    if principal_id in context.owner | {context.creator} | context.contributors:
+    if principal_id in {context.creator} | roles.owner | roles.contributors | roles.managers:
         return True
     # grant access to allowed contributors
-    restrictions = IContributorRestrictions(context).get_restrictions(principal_id)
-    if restrictions and restrictions.check_access(context,
-                                                  permission=MANAGE_CONTENT_PERMISSION,
-                                                  request=request):
-        return True
-    # grant access to local content managers
-    if principal_id in context.managers:
+    restrictions = IContributorRestrictions(context)
+    if restrictions and restrictions.can_access(context,
+                                                permission=MANAGE_CONTENT_PERMISSION,
+                                                request=request):
         return True
     # grant access to shared tool managers if restrictions apply
-    restrictions = IManagerRestrictions(context).get_restrictions(principal_id)
-    return restrictions and restrictions.check_access(context,
-                                                      permission=MANAGE_CONTENT_PERMISSION,
-                                                      request=request)
+    restrictions = IManagerRestrictions(context)
+    return restrictions and restrictions.can_access(context,
+                                                    permission=MANAGE_CONTENT_PERMISSION,
+                                                    request=request)
 
 
 #
@@ -188,6 +202,11 @@ def remove_scheduler_task(context):
             del scheduler[task_id]
 
 
+def reset_publication_action(wf, context):
+    """Refuse version publication"""
+    IWorkflowPublicationInfo(context).reset(complete=True)
+
+
 def prepublish_action(wf, context):
     """Publish content with a future effective publication date
 
@@ -203,7 +222,7 @@ def prepublish_action(wf, context):
         task = ContentPublishingTask(context_id,
                                      prepublished_to_published.transition_id)
         task.name = 'Planned publication for {}'.format(ISequentialIdInfo(context).public_oid)
-        task.schedule_mode = 'Date-style scheduling'
+        task.schedule_mode = SCHEDULER_TASK_DATE_MODE
         pub_info = IWorkflowPublicationInfo(context)
         schedule_info = IDateTaskScheduling(task)
         schedule_info.active = True
@@ -220,18 +239,14 @@ def publish_action(wf, context):
     """Publish version"""
     request = check_request()
     translate = request.localizer.translate
-    now = gmtime(datetime.now(timezone.utc))
     publication_info = IWorkflowPublicationInfo(context)
-    publication_info.publication_date = now
+    publication_info.publication_date = datetime.now(timezone.utc)
     publication_info.publisher = request.principal.id
-    publication_info.apply_first_publication_date()
     version_id = IWorkflowState(context).version_id
-    for version in IWorkflowVersions(context).get_versions((PUBLISHED, )):
+    for version in IWorkflowVersions(context).get_versions((PRE_PUBLISHED, PUBLISHED)):
         if version is not context:
-            IWorkflowInfo(version).fire_transition_toward(ARCHIVED,
-                                                          comment=translate(_("Published version {0}")).format(
-                                                              version_id),
-                                                          check_security=False)
+            IWorkflowInfo(version).fire_transition_toward(
+                ARCHIVED, comment=translate(_("Published version {0}")).format(version_id))
     # check expiration date and create auto-archiving task if needed
     # we compare expiration date with current date to handle the case where content is
     # published automatically at application startup, and we add a small amount of time
@@ -246,8 +261,9 @@ def publish_action(wf, context):
                 del scheduler[task_id]
             task = ContentArchivingTask(context_id)
             task.name = 'Planned archiving for {}'.format(ISequentialIdInfo(context).public_oid)
-            task.schedule_mode = 'Date-style scheduling'
+            task.schedule_mode = SCHEDULER_TASK_DATE_MODE
             pub_info = IWorkflowPublicationInfo(context)
+            now = datetime.now(timezone.utc)
             schedule_info = IDateTaskScheduling(task)
             schedule_info.active = True
             schedule_info.start_date = max(now + timedelta(seconds=10),
@@ -294,11 +310,13 @@ draft_to_prepublished = Transition(transition_id='draft_to_prepublished',
                                    title=_("Pre-publish content"),
                                    source=DRAFT,
                                    destination=PRE_PUBLISHED,
-                                   trigger=SYSTEM,
+                                   trigger=SYSTEM_TRANSITION,
                                    action=prepublish_action,
                                    history_label=_("Content pre-published"),
                                    notify_roles={'*'},
-                                   notify_message=_("pre-published the content « {0} »"))
+                                   notify_title=_("Content publication"),
+                                   notify_message=_("{principal} pre-published the content "
+                                                    "« {title} »"))
 
 prepublished_to_draft = Transition(transition_id='prepublished_to_draft',
                                    title=_("Cancel publication"),
@@ -307,34 +325,38 @@ prepublished_to_draft = Transition(transition_id='prepublished_to_draft',
                                    permission=MANAGE_CONTENT_PERMISSION,
                                    condition=can_manage_content,
                                    action=cancel_prepublish_action,
-                                   menu_css_class='fa fa-fw fa-mail-reply',
+                                   menu_icon_class='fas fa-fw fa-reply',
                                    view_name='wf-cancel-publish.html',
                                    history_label=_("Publication canceled"),
                                    notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
-                                   notify_message=_("cancelled the publication for content « {0} »"),
+                                   notify_title=_("Content publication"),
+                                   notify_message=_("{principal} cancelled the publication "
+                                                    "for content « {title} »"),
                                    order=1)
 
 prepublished_to_published = Transition(transition_id='prepublished_to_published',
                                        title=_("Publish content"),
                                        source=PRE_PUBLISHED,
                                        destination=PUBLISHED,
-                                       trigger=SYSTEM,
+                                       trigger=SYSTEM_TRANSITION,
                                        action=publish_action,
                                        history_label=_("Content published"))
 
 draft_to_published = Transition(transition_id='draft_to_published',
-                                title=_("Publish"),
+                                title=_("Publish content"),
                                 source=DRAFT,
                                 destination=PUBLISHED,
                                 permission=PUBLISH_CONTENT_PERMISSION,
                                 condition=can_manage_content,
                                 action=publish_action,
                                 prepared_transition=draft_to_prepublished,
-                                menu_css_class='fa fa-fw fa-thumbs-o-up',
+                                menu_icon_class='fas fa-fw fa-thumbs-up',
                                 view_name='wf-publish.html',
                                 history_label=_("Content published"),
                                 notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
-                                notify_message=_("published the content « {0} »"),
+                                notify_title=_("Content publication"),
+                                notify_message=_("{principal} published the content "
+                                                 "« {title} »"),
                                 order=1)
 
 published_to_archived = Transition(transition_id='published_to_archived',
@@ -344,11 +366,12 @@ published_to_archived = Transition(transition_id='published_to_archived',
                                    permission=PUBLISH_CONTENT_PERMISSION,
                                    condition=can_manage_content,
                                    action=archive_action,
-                                   menu_css_class='fa fa-fw fa-archive',
+                                   menu_icon_class='fas fa-fw fa-archive',
                                    view_name='wf-archive.html',
+                                   show_operator_warning=True,
                                    history_label=_("Content archived"),
                                    notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
-                                   notify_message=_("archived content « {0} »"),
+                                   notify_message=_("{principal} archived content « {title} »"),
                                    order=2)
 
 published_to_draft = Transition(transition_id='published_to_draft',
@@ -358,7 +381,7 @@ published_to_draft = Transition(transition_id='published_to_draft',
                                 permission=CREATE_VERSION_PERMISSION,
                                 condition=can_create_new_version,
                                 action=clone_action,
-                                menu_css_class='fa fa-fw fa-file-o',
+                                menu_icon_class='fas fa-fw fa-file',
                                 view_name='wf-clone.html',
                                 history_label=_("New version created"),
                                 order=3)
@@ -370,7 +393,7 @@ archived_to_draft = Transition(transition_id='archived_to_draft',
                                permission=CREATE_VERSION_PERMISSION,
                                condition=can_create_new_version,
                                action=clone_action,
-                               menu_css_class='fa fa-fw fa-file-o',
+                               menu_icon_class='fas fa-fw fa-file',
                                view_name='wf-clone.html',
                                history_label=_("New version created"),
                                order=4)
@@ -382,7 +405,7 @@ delete = Transition(transition_id='delete',
                     permission=MANAGE_CONTENT_PERMISSION,
                     condition=can_delete_version,
                     action=delete_action,
-                    menu_css_class='fa fa-fw fa-trash',
+                    menu_icon_class='fas fa-fw fa-trash',
                     view_name='wf-delete.html',
                     history_label=_("Version deleted"),
                     order=99)
@@ -400,15 +423,19 @@ wf_transitions = {init,
 
 @implementer(IBasicWorkflow)
 class BasicWorkflow(Workflow):
-    """PyAMS basic workflow"""
+    """PyAMS basic content workflow"""
+
+    label = _("PyAMS basic workflow")
 
 
-@adapter_config(context=IBasicWorkflow, provides=IWorkflowStateLabel)
-class WorkflowStateLabelAdapter(ContextAdapter):
-    """Generic state label adapter"""
+@adapter_config(required=IBasicWorkflow,
+                provides=IWorkflowStateLabel)
+class BasicWorkflowStateLabelAdapter(ContextAdapter):
+    """Basic workflow state label adapter"""
 
     @staticmethod
     def get_label(content, request=None, format=True):
+        """Workflow state label getter"""
         if request is None:
             request = check_request()
         translate = request.localizer.translate
@@ -417,19 +444,23 @@ class WorkflowStateLabelAdapter(ContextAdapter):
         if header is not None:
             state_label = translate(header)
             if format:
-                state_label = translate(_('{state} {date}')).format(state=state_label,
-                                                                    date=format_datetime(state.state_date))
+                state_label = translate(_('{state} {date}')).format(
+                    state=state_label,
+                    date=format_datetime(state.state_date))
         else:
             state_label = translate(_("Unknown state"))
         return state_label
 
 
-@adapter_config(name=DRAFT, context=IBasicWorkflow, provides=IWorkflowStateLabel)
-class DraftWorkflowStateLabelAdapter(ContextAdapter):
-    """Draft state label adapter"""
+@adapter_config(name=DRAFT,
+                required=IBasicWorkflow,
+                provides=IWorkflowStateLabel)
+class BasicWorkflowDraftStateLabelAdapter(ContextAdapter):
+    """Basic workflow draft state label adapter"""
 
     @staticmethod
     def get_label(content, request=None, format=True):
+        """Workflow state label getter"""
         if request is None:
             request = check_request()
         translate = request.localizer.translate
@@ -444,10 +475,15 @@ class DraftWorkflowStateLabelAdapter(ContextAdapter):
             else:
                 state_label = translate(_("Unknown state"))
         else:
-            state_label = translate(_('publication refused'))
+            history_item = state.history[-1]
+            if history_item.source_state == CANCELED:
+                state_label = translate(_('publication request cancelled'))
+            else:
+                state_label = translate(_('publication refused'))
         if format:
-            state_label = translate(_('{state} {date}')).format(state=state_label,
-                                                                date=format_datetime(state.state_date))
+            state_label = translate(_('{state} {date}')).format(
+                state=state_label,
+                date=format_datetime(state.state_date))
         return state_label
 
 
@@ -466,13 +502,9 @@ wf = BasicWorkflow(wf_transitions,
                    auto_retired_state=ARCHIVED)
 
 
-@utility_config(name='PyAMS basic workflow', provides=IWorkflow)
-class WorkflowUtility(object):
-    """PyAMS basic workflow utility
-
-    This is a basic workflow implementation for PyAMS contents.
-    It only implements three states which are *draft*, *published* and *archived*.
-    """
+@utility_config(name=BASIC_CONTENT_WORKFLOW, provides=IWorkflow)
+class BasicWorkflowUtility:
+    """PyAMS basic workflow utility"""
 
     def __new__(cls):
         return wf
