@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -10,6 +10,10 @@
 # FOR A PARTICULAR PURPOSE.
 #
 
+"""PyAMS_*** module
+
+"""
+
 __docformat__ = 'restructuredtext'
 
 from fanstatic import Library, Resource
@@ -17,10 +21,6 @@ from fanstatic import Library, Resource
 
 library = Library('pyams_content', 'resources')
 
-pyams_content_css = Resource(library, 'css/pyams_content.css',
-                             minified='css/pyams_content.min.css')
-
-pyams_content = Resource(library, 'js/pyams_content.js',
-                         minified='js/pyams_content.min.js',
-                         depends=(pyams_content_css,),
-                         bottom=True)
+content_js = Resource(library, 'js/content.js',
+                      minified='js/content.min.js',
+                      bottom=True)
